-- See page 43 for the schemas.
-- See page 44 for the foreign keys.

DROP TABLE IF EXISTS Student;
CREATE TABLE Student ( Id      INTEGER, 
                       Name    CHAR(20) NOT NULL, 
                       Address CHAR(50), 
                       Status  CHAR(10) DEFAULT 'Freshman',
                       PRIMARY KEY (Id) );                      


DROP TABLE IF EXISTS Professor;
CREATE TABLE Professor ( Id INTEGER,
                         Name CHAR(20) NOT NULL,
                         DeptId CHAR(4),
                         PRIMARY KEY (Id) );
                         

DROP TABLE IF EXISTS Course;                         
CREATE TABLE Course ( CrsCode CHAR(6), 
                      DeptId  CHAR(4), 
                      CrsName CHAR(20), 
                      Descr   CHAR(100),
                      PRIMARY KEY (CrsCode),
                      UNIQUE (DeptId, CrsName) );


-- see page 53 (DOMAIN does not seem to be supported by either SQLite of MySQL)    
--CREATE DOMAIN Grades CHAR(1)
--    CHECK ( VALUE IN ('A', 'B', 'C', 'D', 'F', 'I') );


DROP TABLE IF EXISTS Transcript;
CREATE TABLE Transcript ( StudId   INTEGER, 
                          CrsCode  CHAR(6), 
                          Semester CHAR(6),
                          Grade    CHAR(1),
                          PRIMARY KEY (StudId, CrsCode, Semester),
                          FOREIGN KEY (StudId) REFERENCES Student(Id),
                          FOREIGN KEY (CrsCode) REFERENCES Course,
                          FOREIGN KEY (CrsCode, Semester) REFERENCES Teaching,  
                          CHECK ( Grade IN ('A', 'B', 'C', 'D', 'F', 'I') ),
                          CHECK ( StudId > 0 AND StudId < 1000000000 ) );                          
                         
                         
-- see page 57
DROP TABLE IF EXISTS Teaching;                         
CREATE TABLE Teaching ( ProfId INTEGER, 
                        CrsCode CHAR(6), 
                        Semester CHAR(6),
                        PRIMARY KEY (CrsCode, Semester),
                        FOREIGN KEY (CrsCode) REFERENCES Course(CrsCode)
                             ON DELETE SET NULL
                             ON UPDATE CASCADE,
                        FOREIGN KEY (ProfId)  REFERENCES Professor(Id) 
                             ON DELETE NO ACTION
                             ON UPDATE CASCADE );
                        

-- see pages 45 and 52 (ASSERTION does not seem to be supported by either SQLite of MySQL) 
--CREATE ASSERTION AnInclusionDependency 
--    CHECK ( (SELECT (CrsCode,Semester) FROM Teaching)
--              == (SELECT DISTINCT (CrsCode,Semester) FROM Transcript) )
              
              
-- see page 59 (SQLite seems to have a slightly different syntax for views)
DROP VIEW IF EXISTS ProfStud; 
CREATE VIEW ProfStud (Prof, Stud) AS
SELECT Teaching.ProfId, Transcript.StudId
FROM Transcript, Teaching
WHERE Transcript.CrsCode = Teaching.CrsCode
       AND Transcript.Semester = Teaching.Semester;
       
       

-- see page 14 or page 36
INSERT INTO Student( Id, Name, Address, Status ) 
             VALUES( 111111111, 'John Doe', '123 Main St.', 'Freshman' ); 
INSERT INTO Student( Id, Name, Address, Status ) 
             VALUES( 666666666, 'Joseph Public', '666 Hollow Rd.', 'Sophomore' );
INSERT INTO Student( Id, Name, Address, Status ) 
             VALUES( 111223344, 'Mary Smith', '1 Lake St.', 'Freshman' );
INSERT INTO Student( Id, Name, Address, Status ) 
             VALUES( 987654321, 'Bart Simpson', 'Fox 5 TV', 'Senior' );
INSERT INTO Student( Id, Name, Address, Status ) 
             VALUES( 023456789, 'Homer Simpson', 'Fox 5 TV', 'Senior' );
INSERT INTO Student( Id, Name, Address, Status ) 
             VALUES( 123454321, 'Joe Blow', '6 Yard Ct.', 'Junior' );             


-- Here is one way to get a DEFAULT value into a table
INSERT INTO Student( Id, Name, Address ) 
             VALUES( 123121235, 'Jane Doe', NULL );

-- Here is another way to get a DEFAULT value into a table (doesn't seem to work in SQLite)
INSERT INTO Student( Id, Status, Name, Address ) 
             VALUES( 123121236, DEFAULT, 'Jim Doe', NULL );

SELECT * FROM Student;


-- see page 39
INSERT INTO Professor( Id, Name, DeptId ) 
               VALUES( 101202303, 'John Smyth', 'CS' );
INSERT INTO Professor( Id, Name, DeptId ) 
               VALUES( 783432188, 'Adrian Jones', 'MGT' );
INSERT INTO Professor( Id, Name, DeptId ) 
               VALUES( 121232343, 'David Jones', 'EE' );
INSERT INTO Professor( Id, Name, DeptId ) 
               VALUES( 864297531, 'Qi Chen', 'MAT' );
INSERT INTO Professor( Id, Name, DeptId ) 
               VALUES( 555666777, 'Mary Doe', 'CS' );
INSERT INTO Professor( Id, Name, DeptId ) 
               VALUES( 009406321, 'Jacob Taylor', 'MGT' );
INSERT INTO Professor( Id, Name, DeptId ) 
               VALUES( 900120450, 'Ann White', 'MAT' );
             
SELECT * FROM Professor;


-- see page 39
INSERT INTO Course( CrsCode, DeptId, CrsName, Descr ) 
            VALUES( 'CS305', 'CS', 'Database Systems', 'On the road to high-paying job' );
INSERT INTO Course( CrsCode, DeptId, CrsName, Descr )
            Values( 'CS315', 'CS', 'Transaction Processing', 'Recover from your worst crashes' );
INSERT INTO Course( CrsCode, DeptId, CrsName, Descr )
            Values( 'MGT123', 'MGT', 'Market Analysis', 'Get rich quick' );
INSERT INTO Course( CrsCode, DeptId, CrsName, Descr )
            Values( 'EE101', 'EE', 'Electronic Circuits', 'Build your own computer' );
INSERT INTO Course( CrsCode, DeptId, CrsName, Descr )
            Values( 'MAT123', 'MAT', 'Algebra', 'The world where 2*2!=4' );
            
SELECT * FROM Course;            
            

-- see page 39
INSERT INTO Transcript( StudId, CrsCode, Semester, Grade ) 
                VALUES( '666666666', 'MGT123', 'F1994', 'A' );
INSERT INTO Transcript( StudId, CrsCode, Semester, Grade ) 
                VALUES( '666666666', 'EE101', 'S1991', 'B' );
INSERT INTO Transcript( StudId, CrsCode, Semester, Grade ) 
                VALUES( '666666666', 'MAT123', 'F1997', 'B' );
INSERT INTO Transcript( StudId, CrsCode, Semester, Grade ) 
                VALUES( '987654321', 'CS305', 'F1995', 'C' );
INSERT INTO Transcript( StudId, CrsCode, Semester, Grade ) 
                VALUES( '987654321', 'MGT123', 'F1994', 'B' );
INSERT INTO Transcript( StudId, CrsCode, Semester, Grade ) 
                VALUES( '123454321', 'CS315', 'S1997', 'A' );
INSERT INTO Transcript( StudId, CrsCode, Semester, Grade ) 
                VALUES( '123454321', 'CS305', 'S1996', 'A' );
INSERT INTO Transcript( StudId, CrsCode, Semester, Grade ) 
                VALUES( '123454321', 'MAT123', 'S1996', 'C' );
INSERT INTO Transcript( StudId, CrsCode, Semester, Grade ) 
                VALUES( '023456789', 'EE101', 'F1995', 'B' );
INSERT INTO Transcript( StudId, CrsCode, Semester, Grade ) 
                VALUES( '023456789', 'CS305', 'S1996', 'A' );
INSERT INTO Transcript( StudId, CrsCode, Semester, Grade ) 
                VALUES( '111111111', 'EE101', 'F1997', 'A' );
INSERT INTO Transcript( StudId, CrsCode, Semester, Grade ) 
                VALUES( '111111111', 'MAT123', 'F1997', 'B' );
INSERT INTO Transcript( StudId, CrsCode, Semester, Grade ) 
                VALUES( '111111111', 'MGT123', 'F1997', 'B' );
                
SELECT * FROM Transcript;    



-- see page 40
INSERT INTO Teaching( ProfId, CrsCode, Semester ) 
              VALUES( '009406321', 'MGT123', 'F1994' ); 
INSERT INTO Teaching( ProfId, CrsCode, Semester ) 
              VALUES( '121232343', 'EE101', 'S1991' ); 
INSERT INTO Teaching( ProfId, CrsCode, Semester ) 
              VALUES( '555666777', 'CS305', 'F1995' ); 
INSERT INTO Teaching( ProfId, CrsCode, Semester ) 
              VALUES( '864297531', 'MGT123', 'F1994' ); 
INSERT INTO Teaching( ProfId, CrsCode, Semester ) 
              VALUES( '101202303', 'CS315', 'S1997' ); 
INSERT INTO Teaching( ProfId, CrsCode, Semester ) 
              VALUES( '900120450', 'MAT123', 'S1996' ); 
INSERT INTO Teaching( ProfId, CrsCode, Semester ) 
              VALUES( '121232343', 'EE101', 'F1995' ); 
INSERT INTO Teaching( ProfId, CrsCode, Semester ) 
              VALUES( '101202303', 'CS305', 'S1996' ); 
INSERT INTO Teaching( ProfId, CrsCode, Semester ) 
              VALUES( '900120450', 'MAT123', 'F1997' ); 
INSERT INTO Teaching( ProfId, CrsCode, Semester ) 
              VALUES( '783432188', 'MGT123', 'F1997' ); 
INSERT INTO Teaching( ProfId, CrsCode, Semester ) 
              VALUES( '009406321', 'MGT123', 'F1997' ); 
              
SELECT * FROM Teaching;


SELECT * FROM ProfStud;
